import * as React from 'react';
import { useEffect, useRef, useState } from 'react';
import { throttle } from '../../utils/throttle';
import PropTypes from 'prop-types';

const ScrollSpy = ({
  children,

  // refs
  navContainerRef,
  parentScrollContainerRef,

  // throttle
  scrollThrottle = 300,

  // callback
  onUpdateCallback,

  // offsets
  offsetTop = 0,
  offsetBottom = 0,

  // customize attributes
  useDataAttribute = 'to-scrollspy-id',
  // activeClass = 'active-scroll-spy',

  useBoxMethod = true,
  // updateHistoryStack = true,
}) => {
  const scrollContainerRef = useRef(null);
  const [navContainerItems, setNavContainerItems] = useState(); // prettier-ignore

  // keeps track of the Id in navcontainer which is active
  // so as to not update classLists unless it has been updated
  const prevIdTracker = useRef('');

  // To get the nav container items depending on whether the parent ref for the nav container is passed or not
  useEffect(() => {
    navContainerRef
      ? setNavContainerItems(
          navContainerRef.current?.querySelectorAll(
            `[data-${useDataAttribute}]`,
          ),
        )
      : setNavContainerItems(
          document.querySelectorAll(`[data-${useDataAttribute}]`),
        );
  }, [navContainerRef]);

  // fire once after nav container items are set
  useEffect(() => {
    checkAndUpdateActiveScrollSpy();
  }, [navContainerItems]);

  const isVisible = (el) => {
    const rectInView = el.getBoundingClientRect();

    if (useBoxMethod) {
      const useHeight = parentScrollContainerRef?.current
        ? parentScrollContainerRef?.current.offsetHeight
        : window.innerHeight;
      const hitbox_top = useHeight;
      const element_top = rectInView.top;
      const element_bottom = rectInView.top + useHeight;

      return (
        hitbox_top < element_bottom + offsetBottom &&
        hitbox_top > element_top - offsetTop
      );
    } else {
      // this decides how much of the element should be visible
      const leniency = parentScrollContainerRef?.current
        ? parentScrollContainerRef?.current.offsetHeight * 0.5
        : window.innerHeight * 0.5;

      const useHeight = parentScrollContainerRef?.current
        ? parentScrollContainerRef?.current.offsetHeight
        : window.innerHeight;

      return (
        rectInView.top + leniency + offsetTop >= 0 &&
        rectInView.bottom - leniency - offsetBottom <= useHeight
      );
    }
  };

  const checkAndUpdateActiveScrollSpy = () => {
    const scrollParentContainer = scrollContainerRef.current;

    // if there are no children, return
    if (!(scrollParentContainer && navContainerItems)) return;

    // loop over all children in scroll container
    for (let i = 0; i < scrollParentContainer.children.length; i++) {
      // get child element
      const useChild = scrollParentContainer.children.item(i);
      const elementIsVisible = isVisible(useChild);
      // check if the element is in the viewport
      if (elementIsVisible) {
        const changeHighlightedItemId = useChild.id;
        if (prevIdTracker.current === changeHighlightedItemId) return;
        prevIdTracker.current = changeHighlightedItemId;
        onUpdateCallback && onUpdateCallback(changeHighlightedItemId);
        break;
      }
    }
  };

  useEffect(() => {
    // listen for scroll event
    parentScrollContainerRef
      ? // if ref for scrollable div is provided
        parentScrollContainerRef.current?.addEventListener(
          'scroll',
          throttle(checkAndUpdateActiveScrollSpy, scrollThrottle),
        )
      : // else listen for scroll in window
        window.addEventListener(
          'scroll',
          throttle(checkAndUpdateActiveScrollSpy, scrollThrottle),
        );
  });

  return <div ref={scrollContainerRef}>{children}</div>;
};

ScrollSpy.propTypes = {
  children: PropTypes.node.isRequired,
  navContainerRef: PropTypes.object,
  parentScrollContainerRef: PropTypes.object,
  scrollThrottle: PropTypes.number,
  onUpdateCallback: PropTypes.func,
  offsetTop: PropTypes.number,
  offsetBottom: PropTypes.number,
  useDataAttribute: PropTypes.string,
  activeClass: PropTypes.string,
  useBoxMethod: PropTypes.bool,
  updateHistoryStack: PropTypes.bool,
};

export default ScrollSpy;
