import React, { useState, useEffect } from 'react';
import { useTranslation } from 'react-i18next';
import { Link, scroller } from 'react-scroll';
import { useParams } from 'react-router-dom';
import classNames from 'classnames';

// assets
import logo_zh_white from '../assets/imgs/flyinsono_logos/logo_zh_white.png';
import logo_en_white from '../assets/imgs/flyinsono_logos/logo_en_white.png';

// components
import LanguageList from '../components/public/LanguageList';
import LanguageListTrigger from '../components/public/LanguageListTrigger';
import SectionHome from '../components/home/SectionHome';
import SectionUseCases from '../components/home/SectionUseCases';
import SectionDownload from '../components/home/SectionDownload';
import SectionAbout from '../components/home/SectionAbout';
import RemoteDiagnosis from '../components/home/features/RemoteDiagnosis';
import RemoteConsultation from '../components/home/features/RemoteConsultation';
import RemoteQC from '../components/home/features/RemoteQC';
import MobileRouterTrigger from '../components/public/MobileRouterTrigger';
import MobileRouter from '../components/public/MobileRouter';
import ScrollSpy from '../components/public/ScrollSpy';

export default function HomePage() {
  // static config
  const AUTO_PLAY_DELAY = 3000; // 自动轮播启动延迟时间 ms
  const AUTO_PLAY_INTERVAL = 6000; // 自动轮播间隔时间 ms
  const ROUTER_SCROLL_DURATION = 600; // 路由切换动画时间 ms

  // params
  const { section, language } = useParams();

  // i18n
  const { t, i18n } = useTranslation();

  const [isExpendLanguageList, setIsExpendLanguageList] = useState(false);
  const [isExpendMobileRouter, setIsExpendMobileRouter] = useState(false);
  const [routerIndex, setRouterIndex] = useState(0);
  const [enableAutoPlay, setEnableAutoPlay] = useState(false);
  const [autoPlay, setAutoPlay] = useState(false);
  const [autoPlayIndex, setAutoPlayIndex] = useState(0);
  const [isCN, setIsCN] = useState(true);

  // 用户路由
  const routers = [
    { name: t('首页'), value: 'home' },
    { name: t('应用场景'), value: 'use-cases' },
    { name: t('特色功能'), value: 'feature-a' },
    { name: t('软件下载'), value: 'downloads' },
    { name: t('关于'), value: 'about' },
  ];

  // 自动轮播路由
  const autoPlayRouters = [
    { name: t('首页'), value: 'home' },
    { name: t('应用场景'), value: 'use-cases' },
    { name: t('远程诊断'), value: 'feature-a' },
    { name: t('远程会诊'), value: 'feature-b' },
    { name: t('远程质控'), value: 'feature-c' },
    { name: t('软件下载'), value: 'downloads' },
    { name: t('关于'), value: 'about' },
  ];

  // onInit
  useEffect(() => {
    if (autoPlayRouters.map((e) => e.value).includes(section)) {
      scrollTo(section);
    } else {
      setEnableAutoPlay(true);
    }
    clearParams();
    window.addEventListener('wheel', onMouseWheelScroll);
    window.addEventListener('touchmove', onMobileTouchMove);
    // onDispose
    return () => {
      window.removeEventListener('wheel', onMouseWheelScroll);
      window.removeEventListener('touchmove', onMobileTouchMove);
    };
  }, []);

  useEffect(() => {
    const timer = setTimeout(() => {
      if (enableAutoPlay) {
        setAutoPlay(true);
      }
    }, AUTO_PLAY_DELAY);
    return () => {
      clearTimeout(timer);
    };
  }, [enableAutoPlay]);

  useEffect(() => {
    i18n.on('languageChanged', function (lng) {
      setIsExpendLanguageList(false);
      if (lng === 'zh') {
        setIsCN(true);
      } else {
        setIsCN(false);
      }
    });

    if (!language) {
      i18n.language === 'zh' ? setIsCN(true) : setIsCN(false);
    } else {
      language === 'zh' ? setIsCN(true) : setIsCN(false);
    }
  }, [i18n]);

  // 处理自动轮播: 如果自动轮播开启，则每个 3s 切换一次路由（autoPlayRouters），如果关闭，则清除定时器
  useEffect(() => {
    if (autoPlay) {
      const timer = setTimeout(() => {
        if (autoPlayIndex >= autoPlayRouters.length - 1) {
          setAutoPlayIndex(0);
          scrollTo(autoPlayRouters[0].value);
        } else {
          scrollTo(autoPlayRouters[autoPlayIndex + 1].value);
          setAutoPlayIndex(autoPlayIndex + 1);
        }
      }, AUTO_PLAY_INTERVAL);
      return () => {
        clearTimeout(timer);
      };
    }
  }, [autoPlay, autoPlayIndex]);

  const scrollTo = (element) => {
    scroller.scrollTo(element, {
      duration: ROUTER_SCROLL_DURATION,
      smooth: 'easeInOutQuart', // 使用贝塞尔曲线进行变速
    });
  };

  const onLanguageListTriggerClick = () => {
    if (isExpendMobileRouter) {
      setIsExpendMobileRouter(false);
    }
    setIsExpendLanguageList(!isExpendLanguageList);
  };

  const onMobileRouterTriggerClick = () => {
    if (isExpendLanguageList) {
      setIsExpendLanguageList(false);
    }
    setIsExpendMobileRouter(!isExpendMobileRouter);
  };

  const onMouseWheelScroll = () => {
    handleUserInteract();
  };

  const onMobileTouchMove = () => {
    handleUserInteract();
  };

  const onUserChangeRouter = () => {
    handleUserInteract();
  };

  const handleUserInteract = () => {
    setAutoPlay(false);
    setEnableAutoPlay(false);
    setIsExpendLanguageList(false);
    setIsExpendMobileRouter(false);
  };

  const onScrollSpyCallback = (elementId) => {
    if (elementId.includes('feature')) {
      elementId = 'feature-a';
    }
    const index = routers.findIndex((e) => e.value === elementId);
    setRouterIndex(index);
  };

  // 清理地址栏参数（清理 /#/ 后面的参数）
  const clearParams = () => {
    const url = window.location.href;
    const index = url.indexOf('#');
    if (index > -1) {
      window.history.replaceState({}, 0, url.substring(0, index));
    }
  };

  const mainClass = classNames(
    'flex flex-col items-center justify-between min-h-screen scroll-smooth',
    {
      'theme-cn': isCN,
      'theme-en': !isCN,
    },
  );

  // 路由下标指示器 [通过 index 计算 left-[x] 的值
  const routerIndicatorClass = classNames(
    'hidden md:block absolute w-8 border-white border-2 duration-300 bottom-0 -translate-x-1/2',
    {
      'left-[10%]': routerIndex === 0,
      'left-[30%]': routerIndex === 1,
      'left-[50%]': routerIndex === 2,
      'left-[70%]': routerIndex === 3,
      'left-[90%]': routerIndex === 4,
    },
  );

  return (
    <main className={mainClass}>
      <nav className='animate-slideDownAndFade fixed z-30 w-full bg-blue-500 text-white h-16 shadow-md'>
        <div className='flex justify-between items-center h-full pl-3 md:px-[10%] duration-300'>
          <img
            src={isCN ? logo_zh_white : logo_en_white}
            alt='logo_zh_white'
            className='h-16 py-3 cursor-pointer'
            onClick={() => scrollTo('home')}
          />
          <div className='mx-10 w-full h-full relative'>
            <div className='grid grid-cols-5 w-full h-full'>
              {routers.map((router, index) => {
                return (
                  <Link
                    key={index}
                    to={router.value}
                    duration={ROUTER_SCROLL_DURATION}
                    smooth={true}
                    className='hidden md:flex items-center justify-center text-xl font-medium cursor-pointer hover:text-gray-300'
                    onClick={() => {
                      onUserChangeRouter();
                    }}>
                    {router.name}
                  </Link>
                );
              })}
            </div>
            <div className={routerIndicatorClass}></div>
          </div>
          <LanguageListTrigger onClick={() => onLanguageListTriggerClick()} />
          <MobileRouterTrigger onClick={() => onMobileRouterTriggerClick()} />
        </div>
      </nav>
      <LanguageList
        isExpend={isExpendLanguageList}
        urlLanguageParam={language}
      />
      <MobileRouter
        isExpend={isExpendMobileRouter}
        routers={routers}
        onClickRouter={(value) => {
          scrollTo(value);
          onUserChangeRouter();
        }}
      />
      {/* 首页 */}
      <ScrollSpy
        useBoxMethod={false}
        onUpdateCallback={onScrollSpyCallback}
        scrollThrottle={30}>
        <section
          id='home'
          className='relative w-full h-screen flex items-center justify-center overflow-hidden'>
          <SectionHome t={t} isCN={isCN} />
        </section>
        {/* 应用场景 */}
        <section
          id='use-cases'
          className='relative w-full h-screen flex items-center justify-center'>
          <SectionUseCases t={t} />
        </section>
        {/* 特色功能-远程诊断 */}
        <section
          id='feature-a'
          className='relative w-full h-screen flex items-center justify-center'>
          <RemoteDiagnosis t={t} isCN={isCN} />
        </section>
        {/* 特色功能-远程会诊 */}
        <section
          id='feature-b'
          className='relative w-full h-screen flex items-center justify-center'>
          <RemoteConsultation t={t} isCN={isCN} />
        </section>
        {/* 特色功能-远程质控 */}
        <section
          id='feature-c'
          className='relative w-full h-screen flex items-center justify-center'>
          <RemoteQC t={t} isCN={isCN} />
        </section>
        {/* 软件下载 */}
        <section
          id='downloads'
          className='relative w-full h-screen flex items-center justify-center'>
          <SectionDownload t={t} isCN={isCN} />
        </section>
        {/* 联系我们 */}
        <section
          id='about'
          className='relative w-full h-screen flex items-center justify-center'>
          <SectionAbout t={t} />
        </section>
      </ScrollSpy>
    </main>
  );
}
